//
// Created by philipp on 20.05.18.
//

#include "../include/PlatoonProtocolLib/VehicleFacade.h"

int main(int argc, char ** argv)
{
    using namespace platoonProtocol;

    if (argc < 2)
    {
        std::cerr << "Too few arguments!\n";
        return -1;
    }

    auto ownVehicleId = static_cast<VehicleId>(std::atoi(argv[1]));
    Vehicle::VehicleEndpoints endpoints;

    for (int arg = 2; arg + 1 < argc; arg += 2)
    {
        auto vehicleId = static_cast<VehicleId>(std::atoi(argv[arg]));
        std::string host{argv[arg + 1]};
        endpoints.emplace_back(VehicleEndpoint{vehicleId, host});
    }

    networking::Networking net;
    std::atomic<bool> running{false};
    VehicleFacade leader{Vehicle::Role::LEADER, net, ownVehicleId};
    leader.addVehicleEndpoints(endpoints);

    leader.get()->setOnRunningPlatoonCallback(
        [&running]
        {
            running = true;
            std::cout << "Platoon is running!\n";
        });

    leader.get()->setOnLeavingPlatoonCallback(
        [&running]
        {
            running = false;
            std::cout << "Left platoon!\n";
        });

    leader.createPlatoon();

    while (!running);

    float ps = 0.0f, ipd = 0.0f;

    while (running)
    {
        leader.setPlatoonConfig(PlatoonConfig{ps, ipd});
        ps += 1.0f;
        ipd += 0.1f;
        sleep(1);
    }

    return 0;
}